// See COPYING for license information.
#include <iostream>
#include <fstream>
#include <cstdlib>
#include "defs.h"
#include "CVersionCompare.h"
#include "CConfigFile.h"
#include "CCacheManager.h"

CCacheManager::CCacheManager(const ECacheManagerAction& action)
{
	// Get cache dir and search for packages in it.
	collect_data(CConfigFile::Instance().GetValue("CacheDir")[0]);

	// Prompt for confirmation on the target.
	std::string	_choice;
	while ( _choice.find('y') && _choice.find('n') ) {
		if ( action == EACTION_CLEAR )
			std::cout << VT_COLOR_DOWNGRADE << "Are you sure you want to clear the package cache?" << VT_COLOR_NORMAL << " [y/n] ";
		else	std::cout << VT_COLOR_DOWNGRADE << "Are you sure you want to clean the package cache?" << VT_COLOR_NORMAL << " [y/n] ";
		std::cout.flush();
		std::getline(std::cin, _choice);
	}
	if ( !_choice.find('n') )	return;

	// If we are only cleaning, remove new packages from the list.
	if ( action == EACTION_CLEAN )	unclean();

	remove_packages();	// Do the dirty deed.
}

void CCacheManager::collect_data(const std::string& dir)
{	// Create a list of all the files in the cache.
	system(std::string("find " + dir + " -regex \".*\\.t[glbx]z\" > " + TMPFILE("zpm-cacheclear.tmp")).c_str());

	std::ifstream			_ifcache;
	std::string			_tmpin;
	_ifcache.open(TMPFILE("zpm-cacheclear.tmp").c_str(), std::ios_base::in);
	while ( !_ifcache.eof() && _ifcache.is_open() ) {
		getline(_ifcache, _tmpin);
		if ( _tmpin.length() > 0 ) _Files.push_back(_tmpin);
	}
	_ifcache.close();

	system(std::string("rm -f " + TMPFILE("zpm-cacheclear.tmp")).c_str());
	std::cout << "Found " << _Files.size() << " package(s) in cache.\n";
}

void CCacheManager::remove_packages()
{	// Clear the package cache.
	for ( unsigned int i = 0; i < _Files.size(); i++ ) {
		system(std::string("rm -f " + _Files[i]).c_str());
		std::cout << "Deleted '" << _Files[i] << "'\n";
	}
	char	num[10];
	sprintf(num, "%d", _Files.size());
	WRITELOG("Deleted " + std::string(num)  + " package(s) from the cache");
	std::cout << "Deleted " << _Files.size() << " package(s) from the cache.\n";
}

void CCacheManager::unclean()
{	// Remove packages from the pending list if they are not to be cleaned.
	CVersionCompare	_vcmp;

	// Search for all the old versions of packages.
	std::vector<std::string>	_rems;
	for ( unsigned int i = 0; i < _Files.size(); i++ ) {
		std::string	_curpkg = get_package_name(_Files[i].substr(_Files[i].rfind('/') + 1));
		std::string	_curver = get_package_version(_Files[i].substr(_Files[i].rfind('/') + 1));

		// Search for a package newer than this 'i' one.
		for ( unsigned int k = 0; k < _Files.size(); k++ ) {
			if ( !_Files[k].length() ) continue;
			std::string	_pkg = get_package_name(_Files[k].substr(_Files[k].rfind('/') + 1));
			std::string	_ver = get_package_version(_Files[k].substr(_Files[k].rfind('/') + 1));
			if ( k != i && _curpkg == _pkg && _vcmp.Compare(_ver, _curver) ) {
				_rems.push_back(_Files[i]);
				_Files[i] = "";
				break;
			}
		}
	}
	_Files = _rems;
}

std::string CCacheManager::get_package_name(const std::string& fromstr)
{	// Returns just the package name from an entire string, i.e. "fghfgh" from "fghfgh-10.0.1-i486-62.1.tgz"
	int	_dashcount = 0;
	for ( int i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount < 3 ) continue;
			return fromstr.substr(0, i);
		}
	}
	if ( _dashcount > 0 ) return fromstr.substr(0, fromstr.find('-'));	// Helps out screwed up packages.
	return "null";	// Oh noez!
}

std::string CCacheManager::get_package_version(const std::string& fromstr)
{	// Returns just the version from the package filename, i.e. "10.0.1-i486-62.1" from "fghfgh-10.0.1-i486-62.1.tgz"
	int	_dashcount = 0;

	for ( int i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount == 3 )
				return fromstr.substr(i + 1, (fromstr.length() - i) - 5);
		}
	}
	// Fix for messed up packages like cupswrapperMFC that Atticus pointed out.
	if ( _dashcount > 0 ) return fromstr.substr(fromstr.find('-') + 1, (fromstr.length() - fromstr.find('-')) - 5);
	return "null";	// Hopefully this is never called.
}
